# This file was auto-generated by Fern from our API Definition.

from ..core.client_wrapper import SyncClientWrapper
import typing
from ..core.request_options import RequestOptions
from .types.list_sources_response import ListSourcesResponse
from ..core.pydantic_utilities import parse_obj_as
from ..shared_errors.errors.bad_request import BadRequest
from ..shared_errors.types.bad_request_response import BadRequestResponse
from ..shared_errors.errors.unauthorized import Unauthorized
from ..shared_errors.types.unauthorized_response import UnauthorizedResponse
from ..shared_errors.errors.method_not_allowed import MethodNotAllowed
from ..shared_errors.types.method_not_allowed_response import MethodNotAllowedResponse
from ..shared_errors.errors.rate_limit_exceeded import RateLimitExceeded
from ..shared_errors.types.rate_limit_response import RateLimitResponse
from ..shared_errors.errors.internal_server_error import InternalServerError
from ..shared_errors.types.internal_server_error_response import InternalServerErrorResponse
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from .types.get_source_response import GetSourceResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..shared_errors.errors.not_found import NotFound
from ..shared_errors.types.not_found_response import NotFoundResponse
from ..core.client_wrapper import AsyncClientWrapper


class SourceClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list_sources(
        self,
        *,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListSourcesResponse:
        """
        Returns metadata for all sources that Sayari collects data from

        Parameters
        ----------
        limit : typing.Optional[int]
            A limit on the number of objects to be returned with a range between 1 and 100. Defaults to 100.

        offset : typing.Optional[int]
            Number of results to skip before returning response. Defaults to 0.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListSourcesResponse

        Examples
        --------
        from sayari import Sayari

        client = Sayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.source.list_sources(
            limit=2,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/sources",
            method="GET",
            params={
                "limit": limit,
                "offset": offset,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ListSourcesResponse,
                    parse_obj_as(
                        type_=ListSourcesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequest(
                    typing.cast(
                        BadRequestResponse,
                        parse_obj_as(
                            type_=BadRequestResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise Unauthorized(
                    typing.cast(
                        UnauthorizedResponse,
                        parse_obj_as(
                            type_=UnauthorizedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowed(
                    typing.cast(
                        MethodNotAllowedResponse,
                        parse_obj_as(
                            type_=MethodNotAllowedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise RateLimitExceeded(
                    typing.cast(
                        RateLimitResponse,
                        parse_obj_as(
                            type_=RateLimitResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        InternalServerErrorResponse,
                        parse_obj_as(
                            type_=InternalServerErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_source(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> GetSourceResponse:
        """
        Returns metadata for a source that Sayari collects data from

        Parameters
        ----------
        id : str
            The unique identifier for a source in the database

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetSourceResponse

        Examples
        --------
        from sayari import Sayari

        client = Sayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.source.get_source(
            id="f4396e4b8a41d1fd9f09ea94d2ebedb9",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/source/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    GetSourceResponse,
                    parse_obj_as(
                        type_=GetSourceResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequest(
                    typing.cast(
                        BadRequestResponse,
                        parse_obj_as(
                            type_=BadRequestResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise Unauthorized(
                    typing.cast(
                        UnauthorizedResponse,
                        parse_obj_as(
                            type_=UnauthorizedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFound(
                    typing.cast(
                        NotFoundResponse,
                        parse_obj_as(
                            type_=NotFoundResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowed(
                    typing.cast(
                        MethodNotAllowedResponse,
                        parse_obj_as(
                            type_=MethodNotAllowedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise RateLimitExceeded(
                    typing.cast(
                        RateLimitResponse,
                        parse_obj_as(
                            type_=RateLimitResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        InternalServerErrorResponse,
                        parse_obj_as(
                            type_=InternalServerErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncSourceClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list_sources(
        self,
        *,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListSourcesResponse:
        """
        Returns metadata for all sources that Sayari collects data from

        Parameters
        ----------
        limit : typing.Optional[int]
            A limit on the number of objects to be returned with a range between 1 and 100. Defaults to 100.

        offset : typing.Optional[int]
            Number of results to skip before returning response. Defaults to 0.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListSourcesResponse

        Examples
        --------
        import asyncio

        from sayari import AsyncSayari

        client = AsyncSayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.source.list_sources(
                limit=2,
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/sources",
            method="GET",
            params={
                "limit": limit,
                "offset": offset,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ListSourcesResponse,
                    parse_obj_as(
                        type_=ListSourcesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequest(
                    typing.cast(
                        BadRequestResponse,
                        parse_obj_as(
                            type_=BadRequestResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise Unauthorized(
                    typing.cast(
                        UnauthorizedResponse,
                        parse_obj_as(
                            type_=UnauthorizedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowed(
                    typing.cast(
                        MethodNotAllowedResponse,
                        parse_obj_as(
                            type_=MethodNotAllowedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise RateLimitExceeded(
                    typing.cast(
                        RateLimitResponse,
                        parse_obj_as(
                            type_=RateLimitResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        InternalServerErrorResponse,
                        parse_obj_as(
                            type_=InternalServerErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_source(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetSourceResponse:
        """
        Returns metadata for a source that Sayari collects data from

        Parameters
        ----------
        id : str
            The unique identifier for a source in the database

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetSourceResponse

        Examples
        --------
        import asyncio

        from sayari import AsyncSayari

        client = AsyncSayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.source.get_source(
                id="f4396e4b8a41d1fd9f09ea94d2ebedb9",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/source/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    GetSourceResponse,
                    parse_obj_as(
                        type_=GetSourceResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequest(
                    typing.cast(
                        BadRequestResponse,
                        parse_obj_as(
                            type_=BadRequestResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise Unauthorized(
                    typing.cast(
                        UnauthorizedResponse,
                        parse_obj_as(
                            type_=UnauthorizedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFound(
                    typing.cast(
                        NotFoundResponse,
                        parse_obj_as(
                            type_=NotFoundResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowed(
                    typing.cast(
                        MethodNotAllowedResponse,
                        parse_obj_as(
                            type_=MethodNotAllowedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise RateLimitExceeded(
                    typing.cast(
                        RateLimitResponse,
                        parse_obj_as(
                            type_=RateLimitResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        InternalServerErrorResponse,
                        parse_obj_as(
                            type_=InternalServerErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from .types import (
    BadGatewayResponse,
    BadRequestResponse,
    ConnectionErrorResponse,
    InternalServerErrorResponse,
    MethodNotAllowedResponse,
    NotAcceptableResponse,
    NotFoundResponse,
    RateLimitResponse,
    UnauthorizedResponse,
)
from .errors import (
    BadGateway,
    BadRequest,
    ConnectionError,
    InternalServerError,
    MethodNotAllowed,
    NotAcceptable,
    NotFound,
    RateLimitExceeded,
    Unauthorized,
)

__all__ = [
    "BadGateway",
    "BadGatewayResponse",
    "BadRequest",
    "BadRequestResponse",
    "ConnectionError",
    "ConnectionErrorResponse",
    "InternalServerError",
    "InternalServerErrorResponse",
    "MethodNotAllowed",
    "MethodNotAllowedResponse",
    "NotAcceptable",
    "NotAcceptableResponse",
    "NotFound",
    "NotFoundResponse",
    "RateLimitExceeded",
    "RateLimitResponse",
    "Unauthorized",
    "UnauthorizedResponse",
]



# This file was auto-generated by Fern from our API Definition.

from .embedded_entity import EmbeddedEntity
from .entity_risk import EntityRisk
import typing
from .entity_registration_date import EntityRegistrationDate
from .company_type import CompanyType
from .status import Status
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic


class PsaEntity(EmbeddedEntity):
    """
    The entity that is possibly the same as the target entity.
    """

    risk: EntityRisk
    registration_date: typing.Optional[EntityRegistrationDate] = None
    company_type: typing.Optional[CompanyType] = None
    latest_status: typing.Optional[Status] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

BusinessPurposeStandard = typing.Union[
    typing.Literal[
        "CNAE2",
        "ISIC3",
        "ISIC31",
        "ISIC4",
        "JSIC13",
        "KSIC10",
        "NACE1",
        "NACE2",
        "NAICS2017",
        "SKD2002",
        "SKD2008",
        "TSIC2552",
        "NACEBEL2003",
        "NACEBEL2008",
        "NAF1",
        "NAF2",
        "GCED2011",
        "SCIAN2018",
        "CCNAE2021",
        "CAEM2005",
        "SBI2008",
        "HS",
        "SIC",
        "SSIC2020",
        "PKD2007",
        "ESA2010",
        "ATECO",
        "CAE",
        "TOL2008",
        "ADGM",
        "NIC2008",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

from ...base_types.types.paginated_response import PaginatedResponse
import typing
from .gender_data import GenderData
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic


class GenderInfo(PaginatedResponse):
    """
    A person's gender
    """

    data: typing.List[GenderData]
    next: typing.Optional[typing.Optional[typing.Any]] = None
    offset: typing.Optional[int] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

AddressType = typing.Union[
    typing.Literal["arrival", "departure", "mailing", "physical", "registered", "business"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import typing
from .project_share_on_create import ProjectShareOnCreate
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class CreateProjectRequest(UniversalBaseModel):
    """
    Examples
    --------
    from sayari.project import CreateProjectRequest

    CreateProjectRequest(
        label="Project Alpha",
    )
    """

    label: str
    share: typing.Optional[ProjectShareOnCreate] = pydantic.Field(default=None)
    """
    Specifies access levels available to users in a project within an organization. For comprehensive access, the admin role is recommended.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.api_error import ApiError
from ..types.bad_gateway_response import BadGatewayResponse


class BadGateway(ApiError):
    def __init__(self, body: BadGatewayResponse):
        super().__init__(status_code=502, body=body)



# This file was auto-generated by Fern from our API Definition.

from .get_source_response import GetSourceResponse
from .list_sources_response import ListSourcesResponse
from .source import Source

__all__ = ["GetSourceResponse", "ListSourcesResponse", "Source"]



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import typing
import pydantic
from .business_purpose_standard import BusinessPurposeStandard
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class BusinessPurposeProperties(UniversalBaseModel):
    code: typing.Optional[str] = pydantic.Field(default=None)
    """
    A code
    """

    date: typing.Optional[str] = pydantic.Field(default=None)
    """
    as-of date
    """

    from_date: typing.Optional[str] = pydantic.Field(default=None)
    """
    start date
    """

    standard: typing.Optional[BusinessPurposeStandard] = pydantic.Field(default=None)
    """
    The type of code (e.g., "ISIC4", "NACE1")
    """

    to_date: typing.Optional[str] = pydantic.Field(default=None)
    """
    end date
    """

    value: typing.Optional[str] = pydantic.Field(default=None)
    """
    A text description
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import typing
import pydantic
from ...generated_types.types.both_identifier_types import BothIdentifierTypes
from ...generated_types.types.country import Country
from ...generated_types.types.entities import Entities
from .profile_enum import ProfileEnum
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class ResolutionBody(UniversalBaseModel):
    name: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Entity name
    """

    identifier: typing.Optional[BothIdentifierTypes] = pydantic.Field(default=None)
    """
    Entity identifier. Can be from either the [Identifier Type](/sayari-library/ontology/enumerated-types#identifier-type) or [Weak Identifier Type](/sayari-library/ontology/enumerated-types#weak-identifier-type) enums.
    """

    address: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Entity address
    """

    city: typing.Optional[str] = pydantic.Field(default=None)
    """
    Entity city that contains the provided city name.
    """

    state: typing.Optional[str] = pydantic.Field(default=None)
    """
    Entity state that contains the provided state name.
    """

    country: typing.Optional[typing.List[Country]] = pydantic.Field(default=None)
    """
    Entity country - must be ISO (3166) Trigram i.e., `USA`. See complete list [here](/sayari-library/ontology/enumerated-types#country)
    """

    date_of_birth: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Entity date of birth
    """

    contact: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Entity contact
    """

    type: typing.Optional[typing.List[Entities]] = pydantic.Field(default=None)
    """
    [Entity type](/sayari-library/ontology/entities). If multiple values are passed for any field, the endpoint will match entities with ANY of the values.
    """

    profile: typing.Optional[ProfileEnum] = pydantic.Field(default=None)
    """
    Specifies the search algorithm to use. `corporate` (default) is optimized for accurate entity attribute matching, ideal for business verification. `suppliers` is tailored for matching entities with trade data, suitable for supply chain use cases. `search` mimics /search/entity behavior, best for name-only matches.
    """

    name_min_percentage: typing.Optional[int] = pydantic.Field(default=None)
    """
    Adding this param enables an alternative matching logic. It will set a minimum percentage of tokens needed to match with user input to be considered a "hit". Accepts integers from 0 to 100 inclusive.
    """

    name_min_tokens: typing.Optional[int] = pydantic.Field(default=None)
    """
    Adding this param enables an alternative matching logic. It sets the minimum number of matching tokens the resolved hits need to have in common with the user input to be considered a "hit". Accepts non-negative integers.
    """

    minimum_score_threshold: typing.Optional[int] = pydantic.Field(default=None)
    """
    Specifies the minimum score required to pass, which controls the strictness of the matching threshold. The default value is 77, and tuned for general use-case accuracy. Increase the value for stricter matching, reduce to loosen.
    """

    search_fallback: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Enables a name search fallback when either the corporate or supplier profiles fails to find a match. When invoked, the fallback will make a call similar to /search/entity on name only. By default set to false.
    """

    cutoff_threshold: typing.Optional[int] = pydantic.Field(default=None)
    """
    Specifies the window of similar results returned in the match group. Increase for fewer multiple matches, decrease to open the aperture and allow for more matches. Default is .8
    """

    skip_post_process: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Bypasses the post-processing setps and re-ranking. Useful for debugging. By default set to false, set to true to enable.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
from .entity_response_data import EntityResponseData
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing
import pydantic


class DeleteResourceResponse(UniversalBaseModel):
    """
    Examples
    --------
    from sayari.resource import DeleteResourceResponse, EntityResponseData

    DeleteResourceResponse(
        data=EntityResponseData(
            type="entity",
            id="YWmNKV",
            project="YWmNKV",
            label="HOME DEPOT, U. S. A., INC.",
            created="2024-04-24 14:24:14.07741+00",
            updated="2024-04-24 14:24:14.07741+00",
            updated_by="auth0|5e45bd8caccd890e68147513",
            version=0,
            entity_id="n9SA4T_33tDmLtS8BhC6Zg",
            tag_ids=[],
            case_status="not_assigned",
        ),
    )
    """

    data: EntityResponseData

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
from .risk_value import RiskValue
import typing
from .risk_level import RiskLevel
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class RiskData(UniversalBaseModel):
    value: RiskValue
    metadata: typing.Dict[str, typing.Optional[typing.Any]]
    level: RiskLevel = pydantic.Field()
    """
    The severity of the risk.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from ..generated_types.types.country import Country
from ..generated_types.types.both_identifier_types import BothIdentifierTypes
from ..generated_types.types.entities import Entities
from .types.profile_enum import ProfileEnum
from ..core.request_options import RequestOptions
from .types.resolution_response import ResolutionResponse
from ..core.pydantic_utilities import parse_obj_as
from ..shared_errors.errors.bad_request import BadRequest
from ..shared_errors.types.bad_request_response import BadRequestResponse
from ..shared_errors.errors.unauthorized import Unauthorized
from ..shared_errors.types.unauthorized_response import UnauthorizedResponse
from ..shared_errors.errors.method_not_allowed import MethodNotAllowed
from ..shared_errors.types.method_not_allowed_response import MethodNotAllowedResponse
from ..shared_errors.errors.not_acceptable import NotAcceptable
from ..shared_errors.types.not_acceptable_response import NotAcceptableResponse
from ..shared_errors.errors.rate_limit_exceeded import RateLimitExceeded
from ..shared_errors.types.rate_limit_response import RateLimitResponse
from ..shared_errors.errors.internal_server_error import InternalServerError
from ..shared_errors.types.internal_server_error_response import InternalServerErrorResponse
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from .types.resolution_body import ResolutionBody
from ..core.serialization import convert_and_respect_annotation_metadata
from ..core.client_wrapper import AsyncClientWrapper

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ResolutionClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def resolution(
        self,
        *,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        name: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        address: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        city: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        state: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        country: typing.Optional[typing.Union[Country, typing.Sequence[Country]]] = None,
        identifier: typing.Optional[typing.Union[BothIdentifierTypes, typing.Sequence[BothIdentifierTypes]]] = None,
        date_of_birth: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        contact: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        type: typing.Optional[typing.Union[Entities, typing.Sequence[Entities]]] = None,
        profile: typing.Optional[ProfileEnum] = None,
        name_min_percentage: typing.Optional[int] = None,
        name_min_tokens: typing.Optional[int] = None,
        minimum_score_threshold: typing.Optional[int] = None,
        search_fallback: typing.Optional[bool] = None,
        cutoff_threshold: typing.Optional[int] = None,
        skip_post_process: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ResolutionResponse:
        """
        The resolution endpoints allow users to search for matching entities against a provided list of attributes. The endpoint is similar to the search endpoint, except it's tuned to only return the best match so the client doesn't need to do as much or any post-processing work to filter down results.

        Parameters
        ----------
        limit : typing.Optional[int]
            A limit on the number of objects to be returned with a range between 1 and 10 inclusive. Defaults to 10.

        offset : typing.Optional[int]
            Number of results to skip before returning response. Defaults to 0.

        name : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Entity name

        address : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Entity address. For optimal matching results, it's recommended to concatenate the full address string (street, city, state, postal code).

        city : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Entity city that contains the provided city name.

        state : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Entity state that contains the provided state name.

        country : typing.Optional[typing.Union[Country, typing.Sequence[Country]]]
            Entity country - must be ISO (3166) Trigram i.e., `USA`. See complete list [here](/sayari-library/ontology/enumerated-types#country)

        identifier : typing.Optional[typing.Union[BothIdentifierTypes, typing.Sequence[BothIdentifierTypes]]]
            Entity identifier. Can be from either the [Identifier Type](/sayari-library/ontology/enumerated-types#identifier-type) or [Weak Identifier Type](/sayari-library/ontology/enumerated-types#weak-identifier-type) enums.

        date_of_birth : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Entity date of birth

        contact : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Entity contact

        type : typing.Optional[typing.Union[Entities, typing.Sequence[Entities]]]
            [Entity type](/sayari-library/ontology/entities). If multiple values are passed for any field, the endpoint will match entities with ANY of the values.

        profile : typing.Optional[ProfileEnum]
            Specifies the search algorithm to use. `corporate` (default) is optimized for accurate entity attribute matching, ideal for business verification. `suppliers` is tailored for matching entities with trade data, suitable for supply chain use cases. `search` mimics /search/entity behavior, best for name-only matches.

        name_min_percentage : typing.Optional[int]
            Adding this param enables an alternative matching logic. It will set a minimum percentage of tokens needed to match with user input to be considered a "hit". Accepts integers from 0 to 100 inclusive.

        name_min_tokens : typing.Optional[int]
            Adding this param enables an alternative matching logic. It sets the minimum number of matching tokens the resolved hits need to have in common with the user input to be considered a "hit". Accepts non-negative integers.

        minimum_score_threshold : typing.Optional[int]
            Specifies the minimum score required to pass, which controls the strictness of the matching threshold. The default value is 77, and tuned for general use-case accuracy. Increase the value for stricter matching, reduce to loosen.

        search_fallback : typing.Optional[bool]
            Enables a name search fallback when either the corporate or supplier profiles fails to find a match. When invoked, the fallback will make a call similar to /search/entity on name only. By default set to false.

        cutoff_threshold : typing.Optional[int]
            Specifies the window of similar results returned in the match group. Increase for fewer multiple matches, decrease to open the aperture and allow for more matches. Default is .8

        skip_post_process : typing.Optional[bool]
            Bypasses the post-processing setps and re-ranking. Useful for debugging. By default set to false, set to true to enable.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ResolutionResponse

        Examples
        --------
        from sayari import Sayari

        client = Sayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.resolution.resolution(
            name="Thomas Bangalter",
            address="8 AVENUE RACHEL",
            country="FRA",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/resolution",
            method="GET",
            params={
                "limit": limit,
                "offset": offset,
                "name": name,
                "address": address,
                "city": city,
                "state": state,
                "country": country,
                "identifier": identifier,
                "date_of_birth": date_of_birth,
                "contact": contact,
                "type": type,
                "profile": profile,
                "name_min_percentage": name_min_percentage,
                "name_min_tokens": name_min_tokens,
                "minimum_score_threshold": minimum_score_threshold,
                "search_fallback": search_fallback,
                "cutoff_threshold": cutoff_threshold,
                "skip_post_process": skip_post_process,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ResolutionResponse,
                    parse_obj_as(
                        type_=ResolutionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequest(
                    typing.cast(
                        BadRequestResponse,
                        parse_obj_as(
                            type_=BadRequestResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise Unauthorized(
                    typing.cast(
                        UnauthorizedResponse,
                        parse_obj_as(
                            type_=UnauthorizedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowed(
                    typing.cast(
                        MethodNotAllowedResponse,
                        parse_obj_as(
                            type_=MethodNotAllowedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 406:
                raise NotAcceptable(
                    typing.cast(
                        NotAcceptableResponse,
                        parse_obj_as(
                            type_=NotAcceptableResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise RateLimitExceeded(
                    typing.cast(
                        RateLimitResponse,
                        parse_obj_as(
                            type_=RateLimitResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        InternalServerErrorResponse,
                        parse_obj_as(
                            type_=InternalServerErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def resolution_post(
        self,
        *,
        request: ResolutionBody,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ResolutionResponse:
        """
        The resolution endpoints allow users to search for matching entities against a provided list of attributes. The endpoint is similar to the search endpoint, except it's tuned to only return the best match so the client doesn't need to do as much or any post-processing work to filter down results.

        Parameters
        ----------
        request : ResolutionBody

        limit : typing.Optional[int]
            A limit on the number of objects to be returned with a range between 1 and 10 inclusive. Defaults to 10.

        offset : typing.Optional[int]
            Number of results to skip before returning response. Defaults to 0.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ResolutionResponse

        Examples
        --------
        from sayari import Sayari
        from sayari.resolution import ResolutionBody

        client = Sayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.resolution.resolution_post(
            limit=1,
            request=ResolutionBody(
                name=["Thomas Bangalter"],
                address=["8 AVENUE RACHEL"],
                country=["FRA"],
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/resolution",
            method="POST",
            params={
                "limit": limit,
                "offset": offset,
            },
            json=convert_and_respect_annotation_metadata(object_=request, annotation=ResolutionBody, direction="write"),
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ResolutionResponse,
                    parse_obj_as(
                        type_=ResolutionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequest(
                    typing.cast(
                        BadRequestResponse,
                        parse_obj_as(
                            type_=BadRequestResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise Unauthorized(
                    typing.cast(
                        UnauthorizedResponse,
                        parse_obj_as(
                            type_=UnauthorizedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowed(
                    typing.cast(
                        MethodNotAllowedResponse,
                        parse_obj_as(
                            type_=MethodNotAllowedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 406:
                raise NotAcceptable(
                    typing.cast(
                        NotAcceptableResponse,
                        parse_obj_as(
                            type_=NotAcceptableResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise RateLimitExceeded(
                    typing.cast(
                        RateLimitResponse,
                        parse_obj_as(
                            type_=RateLimitResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        InternalServerErrorResponse,
                        parse_obj_as(
                            type_=InternalServerErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncResolutionClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def resolution(
        self,
        *,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        name: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        address: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        city: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        state: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        country: typing.Optional[typing.Union[Country, typing.Sequence[Country]]] = None,
        identifier: typing.Optional[typing.Union[BothIdentifierTypes, typing.Sequence[BothIdentifierTypes]]] = None,
        date_of_birth: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        contact: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        type: typing.Optional[typing.Union[Entities, typing.Sequence[Entities]]] = None,
        profile: typing.Optional[ProfileEnum] = None,
        name_min_percentage: typing.Optional[int] = None,
        name_min_tokens: typing.Optional[int] = None,
        minimum_score_threshold: typing.Optional[int] = None,
        search_fallback: typing.Optional[bool] = None,
        cutoff_threshold: typing.Optional[int] = None,
        skip_post_process: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ResolutionResponse:
        """
        The resolution endpoints allow users to search for matching entities against a provided list of attributes. The endpoint is similar to the search endpoint, except it's tuned to only return the best match so the client doesn't need to do as much or any post-processing work to filter down results.

        Parameters
        ----------
        limit : typing.Optional[int]
            A limit on the number of objects to be returned with a range between 1 and 10 inclusive. Defaults to 10.

        offset : typing.Optional[int]
            Number of results to skip before returning response. Defaults to 0.

        name : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Entity name

        address : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Entity address. For optimal matching results, it's recommended to concatenate the full address string (street, city, state, postal code).

        city : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Entity city that contains the provided city name.

        state : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Entity state that contains the provided state name.

        country : typing.Optional[typing.Union[Country, typing.Sequence[Country]]]
            Entity country - must be ISO (3166) Trigram i.e., `USA`. See complete list [here](/sayari-library/ontology/enumerated-types#country)

        identifier : typing.Optional[typing.Union[BothIdentifierTypes, typing.Sequence[BothIdentifierTypes]]]
            Entity identifier. Can be from either the [Identifier Type](/sayari-library/ontology/enumerated-types#identifier-type) or [Weak Identifier Type](/sayari-library/ontology/enumerated-types#weak-identifier-type) enums.

        date_of_birth : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Entity date of birth

        contact : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Entity contact

        type : typing.Optional[typing.Union[Entities, typing.Sequence[Entities]]]
            [Entity type](/sayari-library/ontology/entities). If multiple values are passed for any field, the endpoint will match entities with ANY of the values.

        profile : typing.Optional[ProfileEnum]
            Specifies the search algorithm to use. `corporate` (default) is optimized for accurate entity attribute matching, ideal for business verification. `suppliers` is tailored for matching entities with trade data, suitable for supply chain use cases. `search` mimics /search/entity behavior, best for name-only matches.

        name_min_percentage : typing.Optional[int]
            Adding this param enables an alternative matching logic. It will set a minimum percentage of tokens needed to match with user input to be considered a "hit". Accepts integers from 0 to 100 inclusive.

        name_min_tokens : typing.Optional[int]
            Adding this param enables an alternative matching logic. It sets the minimum number of matching tokens the resolved hits need to have in common with the user input to be considered a "hit". Accepts non-negative integers.

        minimum_score_threshold : typing.Optional[int]
            Specifies the minimum score required to pass, which controls the strictness of the matching threshold. The default value is 77, and tuned for general use-case accuracy. Increase the value for stricter matching, reduce to loosen.

        search_fallback : typing.Optional[bool]
            Enables a name search fallback when either the corporate or supplier profiles fails to find a match. When invoked, the fallback will make a call similar to /search/entity on name only. By default set to false.

        cutoff_threshold : typing.Optional[int]
            Specifies the window of similar results returned in the match group. Increase for fewer multiple matches, decrease to open the aperture and allow for more matches. Default is .8

        skip_post_process : typing.Optional[bool]
            Bypasses the post-processing setps and re-ranking. Useful for debugging. By default set to false, set to true to enable.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ResolutionResponse

        Examples
        --------
        import asyncio

        from sayari import AsyncSayari

        client = AsyncSayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.resolution.resolution(
                name="Thomas Bangalter",
                address="8 AVENUE RACHEL",
                country="FRA",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/resolution",
            method="GET",
            params={
                "limit": limit,
                "offset": offset,
                "name": name,
                "address": address,
                "city": city,
                "state": state,
                "country": country,
                "identifier": identifier,
                "date_of_birth": date_of_birth,
                "contact": contact,
                "type": type,
                "profile": profile,
                "name_min_percentage": name_min_percentage,
                "name_min_tokens": name_min_tokens,
                "minimum_score_threshold": minimum_score_threshold,
                "search_fallback": search_fallback,
                "cutoff_threshold": cutoff_threshold,
                "skip_post_process": skip_post_process,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ResolutionResponse,
                    parse_obj_as(
                        type_=ResolutionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequest(
                    typing.cast(
                        BadRequestResponse,
                        parse_obj_as(
                            type_=BadRequestResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise Unauthorized(
                    typing.cast(
                        UnauthorizedResponse,
                        parse_obj_as(
                            type_=UnauthorizedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowed(
                    typing.cast(
                        MethodNotAllowedResponse,
                        parse_obj_as(
                            type_=MethodNotAllowedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 406:
                raise NotAcceptable(
                    typing.cast(
                        NotAcceptableResponse,
                        parse_obj_as(
                            type_=NotAcceptableResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise RateLimitExceeded(
                    typing.cast(
                        RateLimitResponse,
                        parse_obj_as(
                            type_=RateLimitResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        InternalServerErrorResponse,
                        parse_obj_as(
                            type_=InternalServerErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def resolution_post(
        self,
        *,
        request: ResolutionBody,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ResolutionResponse:
        """
        The resolution endpoints allow users to search for matching entities against a provided list of attributes. The endpoint is similar to the search endpoint, except it's tuned to only return the best match so the client doesn't need to do as much or any post-processing work to filter down results.

        Parameters
        ----------
        request : ResolutionBody

        limit : typing.Optional[int]
            A limit on the number of objects to be returned with a range between 1 and 10 inclusive. Defaults to 10.

        offset : typing.Optional[int]
            Number of results to skip before returning response. Defaults to 0.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ResolutionResponse

        Examples
        --------
        import asyncio

        from sayari import AsyncSayari
        from sayari.resolution import ResolutionBody

        client = AsyncSayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.resolution.resolution_post(
                limit=1,
                request=ResolutionBody(
                    name=["Thomas Bangalter"],
                    address=["8 AVENUE RACHEL"],
                    country=["FRA"],
                ),
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/resolution",
            method="POST",
            params={
                "limit": limit,
                "offset": offset,
            },
            json=convert_and_respect_annotation_metadata(object_=request, annotation=ResolutionBody, direction="write"),
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ResolutionResponse,
                    parse_obj_as(
                        type_=ResolutionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequest(
                    typing.cast(
                        BadRequestResponse,
                        parse_obj_as(
                            type_=BadRequestResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise Unauthorized(
                    typing.cast(
                        UnauthorizedResponse,
                        parse_obj_as(
                            type_=UnauthorizedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowed(
                    typing.cast(
                        MethodNotAllowedResponse,
                        parse_obj_as(
                            type_=MethodNotAllowedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 406:
                raise NotAcceptable(
                    typing.cast(
                        NotAcceptableResponse,
                        parse_obj_as(
                            type_=NotAcceptableResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise RateLimitExceeded(
                    typing.cast(
                        RateLimitResponse,
                        parse_obj_as(
                            type_=RateLimitResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        InternalServerErrorResponse,
                        parse_obj_as(
                            type_=InternalServerErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import pydantic
import typing
from ...generated_types.types.country import Country
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class RecordDetails(UniversalBaseModel):
    id: str = pydantic.Field()
    """
    The unique identifier for a record in the database
    """

    label: str
    source: str = pydantic.Field()
    """
    The unique identifier for a source in the database
    """

    publication_date: typing.Optional[str] = pydantic.Field(default=None)
    """
    The date the record was published
    """

    acquisition_date: str = pydantic.Field()
    """
    The date Sayari acquired this record
    """

    references_count: int = pydantic.Field()
    """
    Number of times this record is referenced
    """

    record_url: str = pydantic.Field()
    """
    The url to download the record from Sayari
    """

    source_url: typing.Optional[str] = pydantic.Field(default=None)
    """
    The url to access to original source
    """

    document_urls: typing.Optional[typing.List[str]] = None
    matches: typing.Optional[typing.Dict[str, typing.List[str]]] = None
    country: typing.Optional[Country] = None
    page: typing.Optional[float] = None
    page_count: typing.Optional[float] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import typing
from .source_id import SourceId
import pydantic
from ...generated_types.types.country import Country
from ...generated_types.types.entities import Entities
from ...generated_types.types.risk import Risk
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class FilterList(UniversalBaseModel):
    """
    Filter your search on the following attributes.
    """

    source: typing.Optional[typing.List[SourceId]] = pydantic.Field(default=None)
    """
    List of source IDs to filter by.
    """

    country: typing.Optional[typing.List[Country]] = pydantic.Field(default=None)
    """
    List of countries to filter by.
    """

    state: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    List of states or provinces to filter by.
    """

    city: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    List of cities to filter by.
    """

    entity_type: typing.Optional[typing.List[Entities]] = pydantic.Field(default=None)
    """
    List of entity types to filter by.
    """

    bounds: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Pipe-delimited bounding box coordinates (north,west,south,east). E.g., '"46.12|-76|45|-75"'.
    """

    risk: typing.Optional[typing.List[Risk]] = pydantic.Field(default=None)
    """
    List of risk factors to filter by.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...base_types.types.paginated_response import PaginatedResponse
import typing
from .additional_information_data import AdditionalInformationData
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic


class AdditionalInformationInfo(PaginatedResponse):
    """
    This is a generic attribute used to hold miscellaneous information not covered by any other attribute. Includes "value" (for the attribute itself), "type" (a name: e.g., "Real property description"), and "extra" (a miscellaneous field to hold any other details).
    """

    data: typing.List[AdditionalInformationData]
    next: typing.Optional[typing.Optional[typing.Any]] = None
    offset: typing.Optional[int] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.api_error import ApiError
from ..types.internal_server_error_response import InternalServerErrorResponse


class InternalServerError(ApiError):
    def __init__(self, body: InternalServerErrorResponse):
        super().__init__(status_code=500, body=body)



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations
from ...base_types.types.paginated_response import PaginatedResponse
from ...shared_types.types.entity_details import EntityDetails
from ...shared_types.types.entity_relationships import EntityRelationships
from ...shared_types.types.relationship_data import RelationshipData
import typing
from .supplier_or_buyer import SupplierOrBuyer
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic
from ...core.pydantic_utilities import update_forward_refs


class SupplierSearchResponse(PaginatedResponse):
    """
    OK

    Examples
    --------
    from sayari.base_types import QualifiedCount
    from sayari.shared_types import Identifier, RiskData, SourceCountInfo, Status
    from sayari.trade import (
        HsCode,
        SupplierMetadata,
        SupplierOrBuyer,
        SupplierSearchResponse,
    )

    SupplierSearchResponse(
        offset=0,
        limit=1,
        size=QualifiedCount(
            count=4,
            qualifier="eq",
        ),
        next=True,
        data=[
            SupplierOrBuyer(
                id="yNwunHdFInERKig0Thusgg",
                label="ERBE ELEKTROMEDIZIN GMBH",
                degree=6349,
                entity_url="/v1/entity/yNwunHdFInERKig0Thusgg",
                pep=False,
                psa_id="4904853096420",
                psa_count=1,
                sanctioned=False,
                closed=False,
                company_type="Foreign entity",
                registration_date="Registered 1965-07-20",
                latest_status=Status(
                    status="active",
                    date="2015-01-07",
                ),
                trade_count={"sent": 6278, "received": 5},
                type="company",
                identifiers=[
                    Identifier(
                        value="52990027L4WYH1AZ2T91",
                        type="lei",
                        label="Lei",
                    ),
                    Identifier(
                        value="26.035.445/0001-09",
                        type="bra_cnpj",
                        label="Bra Cnpj",
                    ),
                    Identifier(
                        value="B8534_HRB380137",
                        type="deu_registernummer",
                        label="Deu Registernummer",
                    ),
                    Identifier(
                        value="0507.995.037",
                        type="bel_enterprise_number",
                        label="Bel Enterprise Number",
                    ),
                    Identifier(
                        value="LBB853438013700000",
                        type="validatis_number",
                        label="Validatis Number",
                    ),
                ],
                addresses=[
                    "WALDHOERNLESTRASSE 17 72072 TUEBINGEN 0 GERMANY",
                    "72072, , TUEBINGEN, WALDHOERNLESTRASSE 17,",
                    "WALDHÖRNLESTRAßE 17",
                ],
                countries=[
                    "USA",
                    "SWE",
                    "BEL",
                    "MEX",
                    "ITA",
                    "SGP",
                    "ECU",
                    "DEU",
                    "BRA",
                    "CAN",
                ],
                relationship_count={
                    "linked_to": 2,
                    "officer_of": 1,
                    "has_officer": 5,
                    "shareholder_of": 3,
                    "has_shareholder": 1,
                    "has_director": 1,
                    "has_lawyer": 1,
                    "owner_of": 20,
                    "ships_to": 30,
                    "receives_from": 3,
                    "subsidiary_of": 1,
                    "has_subsidiary": 1,
                    "has_legal_representative": 5,
                },
                source_count={
                    "ee100f9b5dfdae8991ba43f5de6e1854": SourceCountInfo(
                        count=8,
                        label="Panama Imports & Exports (January 2022 - Present)",
                    ),
                    "26a8072830039f470287902c1530ef79": SourceCountInfo(
                        count=6,
                        label="Belgium Crossroads Bank for Enterprises",
                    ),
                    "c7c2684a1a28cce25025c6b7c6a97bf1": SourceCountInfo(
                        count=24,
                        label="Colombia Imports & Exports (2008-2020)",
                    ),
                    "d302aa0d7153d87aeaa4406f9a3b28fa": SourceCountInfo(
                        count=109,
                        label="Colombia Imports & Exports (January 2022 - Present)",
                    ),
                    "9615bab28dddcc89548c928ab192ee7c": SourceCountInfo(
                        count=11,
                        label="Sri Lanka Imports & Exports (January 2023 - Present)",
                    ),
                    "c22aabd1653affa3d007fb01dcc66b72": SourceCountInfo(
                        count=3,
                        label="Singapore Handshakes Shareholder Profiles (3rd Party Data)",
                    ),
                    "33ce74530696fdc957af6e05194905a2": SourceCountInfo(
                        count=1,
                        label="Nicaragua Imports & Exports (January 2024 - Present)",
                    ),
                    "16279c76cf40cfb1f5ad0809e6a76fd7": SourceCountInfo(
                        count=2,
                        label="Italy Company Registry (3rd Party Data)",
                    ),
                    "4a34a8375e7041cf8661fa82366c60ea": SourceCountInfo(
                        count=161,
                        label="Ecuador Imports & Exports (January 2022 - Present)",
                    ),
                    "8511f57f6098dcc36e3a814c3ea5bdd7": SourceCountInfo(
                        count=3,
                        label="Uzbekistan Imports & Exports (January 2022 - Present)",
                    ),
                    "b812677c0a32a1746b3ac741c7b97ae0": SourceCountInfo(
                        count=58,
                        label="Legal Entity Identifier (LEI) Registry (3rd Party Data)",
                    ),
                    "ac1fa195f9cd4ccf657bca3c6db0bb19": SourceCountInfo(
                        count=1,
                        label="USA Patents and Trademark Office Trademark Applications",
                    ),
                    "7a92887d4f18fc21abe0d658b25364e7": SourceCountInfo(
                        count=735,
                        label="USA USASpending.gov Profiles Database",
                    ),
                    "a7829d9c8999fcaa78af55886da9be06": SourceCountInfo(
                        count=42,
                        label="Turkey Imports & Exports (January 2024 - Present)",
                    ),
                    "92fbab1cd3a92b583e0ec0ca5c48a254": SourceCountInfo(
                        count=1,
                        label="Germany Handelsregister Announcements",
                    ),
                    "ce462e9deea545cce35df38c48512a0c": SourceCountInfo(
                        count=5243,
                        label="India Imports & Exports (January 2023 - Present)",
                    ),
                    "66dfefb726ae00fde8f09f34c5578d35": SourceCountInfo(
                        count=227,
                        label="Russia Imports & Exports (January 2023 - Present)",
                    ),
                    "83ffee6075e54030397fa4c499a5a7ba": SourceCountInfo(
                        count=4,
                        label="Tanzania Imports & Exports (January 2023 - Present)",
                    ),
                    "5547652362cfeeb62963e058f95d6fdd": SourceCountInfo(
                        count=66,
                        label="Peru Imports & Exports (January 2023 - Present)",
                    ),
                    "e5de7b52cc88ef4cd1a10e201bdf46ee": SourceCountInfo(
                        count=71,
                        label="Vietnam Imports & Exports (January 2023 - Present)",
                    ),
                    "b9d809b02049993ba8dc2e4c5f7cceca": SourceCountInfo(
                        count=152,
                        label="Mexico Industrial Property Trademark Registry (MARCIA)",
                    ),
                    "31df3cc3a57cc4a39dcc29dd4b9ce5e5": SourceCountInfo(
                        count=108,
                        label="Mexico Imports & Exports (January 2022 - Present)",
                    ),
                    "31d2e44e0e61ae41a8a7d09b8499fd4e": SourceCountInfo(
                        count=1,
                        label="Belgium KBO Open Data (Active Entities)",
                    ),
                    "b2695e88b6f5b89ba1d9011e06dbcbc7": SourceCountInfo(
                        count=4,
                        label="Botswana Imports & Exports (January 2023 - Present)",
                    ),
                    "f5fdf9a3e7f4ae43eae3191eea1f3127": SourceCountInfo(
                        count=1,
                        label="Sweden Business Register (Statistics Sweden)",
                    ),
                    "9c66f2f4dc556f40ce096ebf4f0c97ac": SourceCountInfo(
                        count=7,
                        label="Germany Validatis Shareholder Lists (3rd Party Data)",
                    ),
                    "16a4cc2d0f467fa993b28587d542a25d": SourceCountInfo(
                        count=398,
                        label="USA Imports (2021 - Present)",
                    ),
                    "a4475601393f5a6a27154175556f7a85": SourceCountInfo(
                        count=2,
                        label="Canada Intellectual Property Office Trademarks Database",
                    ),
                    "fecea39fdc2bbbca36ed77ff68c8c89b": SourceCountInfo(
                        count=32,
                        label="Brazil Receita Federal",
                    ),
                    "96c06a5a03b61b91324c7e05b3114fb6": SourceCountInfo(
                        count=3,
                        label="Ukraine Imports & Exports (January 2023 - May 2023)",
                    ),
                    "0b325d9555f1011ae80446f3778bc3c8": SourceCountInfo(
                        count=1,
                        label="Paraguay Imports & Exports (January 2022 - Present)",
                    ),
                    "41c82cb8bda87edc819b7790081e1e26": SourceCountInfo(
                        count=12,
                        label="Ghana Imports (January 2023 - Present)",
                    ),
                    "9f22a5470e1faf82004e203d094cca29": SourceCountInfo(
                        count=4,
                        label="Kenya Imports (January 2024 - present)",
                    ),
                    "a6cd51a15ff02fb3ab273f3c90d2097c": SourceCountInfo(
                        count=2,
                        label="Italy Chambers of Commerce Registry",
                    ),
                    "b7593dcf982bcb9ca86f24ef369ed1e5": SourceCountInfo(
                        count=145,
                        label="Philippines Imports & Exports (January 2023 - Present)",
                    ),
                },
                risk={
                    "basel_aml": RiskData(
                        value=5.07,
                        metadata={"country": ["ECU"]},
                        level="relevant",
                    ),
                    "exports_bis_high_priority_items_critical_components": RiskData(
                        value=1.0,
                        metadata={"origin_shipment_product": ["854231"]},
                        level="high",
                    ),
                    "cpi_score": RiskData(
                        value=34.0,
                        metadata={"country": ["ECU"]},
                        level="relevant",
                    ),
                    "exports_bis_high_priority_items": RiskData(
                        value=1.0,
                        metadata={"origin_shipment_product": ["853669"]},
                        level="elevated",
                    ),
                },
                user_attribute_counts={},
                user_attribute_count={},
                user_record_count=0,
                user_related_entities_count=0,
                user_relationship_count={},
                related_entities_count=6349,
                attribute_counts={
                    "company_type": 6,
                    "name": 2,
                    "business_purpose": 11,
                    "identifier": 8,
                    "additional_information": 2,
                    "country": 22,
                    "contact": 3,
                    "status": 5,
                    "address": 20,
                    "financials": 1,
                },
                attribute_count={
                    "company_type": 6,
                    "name": 2,
                    "business_purpose": 11,
                    "identifier": 8,
                    "additional_information": 2,
                    "country": 22,
                    "contact": 3,
                    "status": 5,
                    "address": 20,
                    "financials": 1,
                },
                reference_id="fecea39fdc2bbbca36ed77ff68c8c89b/27709659/1566236147596:c14ee31028469ae2871467bb07abc96a",
                metadata=SupplierMetadata(
                    latest_shipment_date="2024-07-10",
                    shipments=6,
                    hs_codes=[
                        HsCode(
                            key="854231",
                            doc_count=6,
                            value="Electronic integrated circuits; processors and controllers, whether or not combined with memories, converters, logic circuits, amplifiers, clock and timing circuits, or other circuits",
                            value_simple="Integrated Circuits",
                        )
                    ],
                ),
            )
        ],
    )
    """

    offset: int
    next: bool
    data: typing.List[SupplierOrBuyer]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


update_forward_refs(EntityDetails, SupplierSearchResponse=SupplierSearchResponse)
update_forward_refs(EntityRelationships, SupplierSearchResponse=SupplierSearchResponse)
update_forward_refs(RelationshipData, SupplierSearchResponse=SupplierSearchResponse)



# This file was auto-generated by Fern from our API Definition.

from .types import (
    EntityId,
    HsCodeWithDescription,
    TradeTraversalEntity,
    TradeTraversalPath,
    TradeTraversalPathSegment,
    TradeTraversalProduct,
    UpstreamTradeTraversalResponse,
)

__all__ = [
    "EntityId",
    "HsCodeWithDescription",
    "TradeTraversalEntity",
    "TradeTraversalPath",
    "TradeTraversalPathSegment",
    "TradeTraversalProduct",
    "UpstreamTradeTraversalResponse",
]



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import typing
from .resource_notification_data import ResourceNotificationData
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic


class ResourceNotificationsResponse(UniversalBaseModel):
    """
    OK

    Examples
    --------
    from sayari.notifications import (
        ResourceNotificationData,
        ResourceNotificationsResponse,
    )

    ResourceNotificationsResponse(
        offset=0,
        limit=20,
        next=False,
        data=[
            ResourceNotificationData(
                saved_resource_id="03ePyj",
                project_id="0oZnoG",
                entity_id="wxwqZshCF4trlrmOa2eu9w",
                type="risk",
                field="forced_labor_sheffield_hallam_university_reports_origin_subtier",
                value="3",
                date="2024-02-15T00:00:00.000Z",
            )
        ],
    )
    """

    offset: int
    limit: int
    next: bool
    data: typing.List[ResourceNotificationData]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
from .resolution_response_fields import ResolutionResponseFields
import typing
from .resolution_result import ResolutionResult
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic


class ResolutionResponse(UniversalBaseModel):
    """
    Examples
    --------
    from sayari.resolution import (
        MatchExplanation,
        MatchStrength,
        ResolutionResponse,
        ResolutionResponseFields,
        ResolutionResult,
    )
    from sayari.shared_types import Identifier

    ResolutionResponse(
        fields=ResolutionResponseFields(
            name=["Thomas Bangalter"],
            address=["8 AVENUE RACHEL"],
            country=["FRA"],
        ),
        data=[
            ResolutionResult(
                profile="corporate",
                score=453.80554,
                entity_id="1nOeH5G2EhmRVtmeVqO2Lw",
                label="Mr Thomas Bangalter",
                type="person",
                identifiers=[
                    Identifier(
                        type="uk_person_number",
                        value="053673450003",
                        label="Uk Person Number",
                    ),
                    Identifier(
                        type="uk_person_number",
                        value="053673450002",
                        label="Uk Person Number",
                    ),
                ],
                addresses=[
                    "5TH FLOOR 104 OXFORD STREET, W1D 1LP, LONDON, UNITED KINGDOM",
                    "Oxford Street, London, W1D 1LP",
                    "8 AVENUE RACHEL, 75018, FRANCE",
                    "4th Floor, 205 Wardour Street, London, W1F 8ZJ",
                    "MSE BUSINESS MANAGEMENT LLP 4TH FLOOR, 205 WARDOUR STREET, W1F 8ZJ, UNITED KINGDOM",
                ],
                countries=["FRA", "GBR"],
                sources=[
                    "ecdfb3f2ecc8c3797e77d5795a8066ef",
                    "4ea8bac1bed868e1510ffd21842e9551",
                ],
                typed_matched_queries=[
                    "address|sub|phraseStripped|0",
                    "name|100match|0",
                    "address|subclause|0",
                    "address|numericFirst|0",
                    "address|numericFull|0",
                    "address|V2|Strict|0",
                    "name|StrictV2|0",
                    "address|V3|0",
                    "name|100phrase|0",
                    "country|0",
                    "address|sub|cleanedAddress|0",
                    "address|sub|phraseFirstHalf|0",
                ],
                matched_queries=["address", "name", "identifier", "country"],
                highlight={
                    "name": ["Mr <em>Thomas</em> <em>Bangalter</em>"],
                    "address": [
                        "<em>8</em> <em>AVENUE</em> <em>RACHEL</em>, 75018, FRANCE"
                    ],
                    "country": ["<em>FRA</em>"],
                },
                explanation={
                    "name": [
                        MatchExplanation(
                            matched="Mr <em>Thomas</em> <em>Bangalter</em>",
                            uploaded="Thomas Bangalter",
                            high_quality_match_name=True,
                            scores={
                                "tf": 0.35648895767689204,
                                "lv": 0.9142857142857143,
                                "fz": 0.8421052631578947,
                                "l1": 1.0,
                                "l2": 1.0,
                            },
                            n_common_term_matches=1,
                            n_uncommon_term_matches=1,
                        )
                    ],
                    "address": [
                        MatchExplanation(
                            matched="<em>8</em> <em>AVENUE</em> <em>RACHEL</em>, 75018, FRANCE",
                            uploaded="8 AVENUE RACHEL",
                            match_quality="medium",
                            scores={"9p": 0.47058823529411764},
                        )
                    ],
                    "country": [
                        MatchExplanation(
                            matched="<em>FRA</em>",
                            uploaded="FRA",
                        )
                    ],
                },
                match_strength=MatchStrength(
                    value="strong",
                ),
            )
        ],
    )
    """

    fields: ResolutionResponseFields
    data: typing.List[ResolutionResult]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .shortest_path_data import ShortestPathData
from .shortest_path_response import ShortestPathResponse
from .traversal_data import TraversalData
from .traversal_path import TraversalPath
from .traversal_relationship_data import TraversalRelationshipData
from .traversal_response import TraversalResponse

__all__ = [
    "ShortestPathData",
    "ShortestPathResponse",
    "TraversalData",
    "TraversalPath",
    "TraversalRelationshipData",
    "TraversalResponse",
]



# This file was auto-generated by Fern from our API Definition.

from .types import (
    MatchExplanation,
    MatchQuality,
    MatchStrength,
    ProfileEnum,
    ResolutionBody,
    ResolutionResponse,
    ResolutionResponseFields,
    ResolutionResult,
)

__all__ = [
    "MatchExplanation",
    "MatchQuality",
    "MatchStrength",
    "ProfileEnum",
    "ResolutionBody",
    "ResolutionResponse",
    "ResolutionResponseFields",
    "ResolutionResult",
]



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from .types.trade_filter_list import TradeFilterList
from ..core.request_options import RequestOptions
from .types.shipment_search_response import ShipmentSearchResponse
from ..core.serialization import convert_and_respect_annotation_metadata
from ..core.pydantic_utilities import parse_obj_as
from ..shared_errors.errors.bad_request import BadRequest
from ..shared_errors.types.bad_request_response import BadRequestResponse
from ..shared_errors.errors.unauthorized import Unauthorized
from ..shared_errors.types.unauthorized_response import UnauthorizedResponse
from ..shared_errors.errors.method_not_allowed import MethodNotAllowed
from ..shared_errors.types.method_not_allowed_response import MethodNotAllowedResponse
from ..shared_errors.errors.rate_limit_exceeded import RateLimitExceeded
from ..shared_errors.types.rate_limit_response import RateLimitResponse
from ..shared_errors.errors.internal_server_error import InternalServerError
from ..shared_errors.types.internal_server_error_response import InternalServerErrorResponse
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from .types.supplier_search_response import SupplierSearchResponse
from .types.buyer_search_response import BuyerSearchResponse
from ..core.client_wrapper import AsyncClientWrapper

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class TradeClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def search_shipments(
        self,
        *,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        q: typing.Optional[str] = OMIT,
        filter: typing.Optional[TradeFilterList] = OMIT,
        facets: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ShipmentSearchResponse:
        """
        Search for a shipment. Please note, searches are limited to a maximum of 10,000 results.

        Parameters
        ----------
        limit : typing.Optional[int]
            A limit on the number of objects to be returned with a range between 1 and 10000. Defaults to 100.

        offset : typing.Optional[int]
            Number of results to skip before returning response. Defaults to 0.

        q : typing.Optional[str]
            Query term. The syntax for the query parameter follows elasticsearch simple query string syntax. The includes the ability to use search operators and to perform nested queries. Must be url encoded.

        filter : typing.Optional[TradeFilterList]
            Filters to be applied to search query to limit the result-set.

        facets : typing.Optional[bool]
            Whether or not to return search facets in results giving counts by field. Defaults to false.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ShipmentSearchResponse

        Examples
        --------
        from sayari import Sayari
        from sayari.trade import TradeFilterList

        client = Sayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.trade.search_shipments(
            limit=1,
            filter=TradeFilterList(
                departure_country=["DEU"],
                arrival_country=["RUS"],
                hs_code=["854231"],
                arrival_date=["2024-01 TO 2024-10"],
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/trade/search/shipments",
            method="POST",
            params={
                "limit": limit,
                "offset": offset,
            },
            json={
                "q": q,
                "filter": convert_and_respect_annotation_metadata(
                    object_=filter, annotation=TradeFilterList, direction="write"
                ),
                "facets": facets,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ShipmentSearchResponse,
                    parse_obj_as(
                        type_=ShipmentSearchResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequest(
                    typing.cast(
                        BadRequestResponse,
                        parse_obj_as(
                            type_=BadRequestResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise Unauthorized(
                    typing.cast(
                        UnauthorizedResponse,
                        parse_obj_as(
                            type_=UnauthorizedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowed(
                    typing.cast(
                        MethodNotAllowedResponse,
                        parse_obj_as(
                            type_=MethodNotAllowedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise RateLimitExceeded(
                    typing.cast(
                        RateLimitResponse,
                        parse_obj_as(
                            type_=RateLimitResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        InternalServerErrorResponse,
                        parse_obj_as(
                            type_=InternalServerErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def search_suppliers(
        self,
        *,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        q: typing.Optional[str] = OMIT,
        filter: typing.Optional[TradeFilterList] = OMIT,
        facets: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SupplierSearchResponse:
        """
        Search for a supplier. Please note, searches are limited to a maximum of 10,000 results.

        Parameters
        ----------
        limit : typing.Optional[int]
            A limit on the number of objects to be returned with a range between 1 and 10000. Defaults to 100.

        offset : typing.Optional[int]
            Number of results to skip before returning response. Defaults to 0.

        q : typing.Optional[str]
            Query term. The syntax for the query parameter follows elasticsearch simple query string syntax. The includes the ability to use search operators and to perform nested queries. Must be url encoded.

        filter : typing.Optional[TradeFilterList]
            Filters to be applied to search query to limit the result-set.

        facets : typing.Optional[bool]
            Whether or not to return search facets in results giving counts by field. Defaults to false.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SupplierSearchResponse

        Examples
        --------
        from sayari import Sayari
        from sayari.trade import TradeFilterList

        client = Sayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.trade.search_suppliers(
            limit=1,
            filter=TradeFilterList(
                departure_country=["DEU"],
                arrival_country=["RUS"],
                hs_code=["854231"],
                arrival_date=["2024-01 TO 2024-10"],
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/trade/search/suppliers",
            method="POST",
            params={
                "limit": limit,
                "offset": offset,
            },
            json={
                "q": q,
                "filter": convert_and_respect_annotation_metadata(
                    object_=filter, annotation=TradeFilterList, direction="write"
                ),
                "facets": facets,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    SupplierSearchResponse,
                    parse_obj_as(
                        type_=SupplierSearchResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequest(
                    typing.cast(
                        BadRequestResponse,
                        parse_obj_as(
                            type_=BadRequestResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise Unauthorized(
                    typing.cast(
                        UnauthorizedResponse,
                        parse_obj_as(
                            type_=UnauthorizedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowed(
                    typing.cast(
                        MethodNotAllowedResponse,
                        parse_obj_as(
                            type_=MethodNotAllowedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise RateLimitExceeded(
                    typing.cast(
                        RateLimitResponse,
                        parse_obj_as(
                            type_=RateLimitResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        InternalServerErrorResponse,
                        parse_obj_as(
                            type_=InternalServerErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def search_buyers(
        self,
        *,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        q: typing.Optional[str] = OMIT,
        filter: typing.Optional[TradeFilterList] = OMIT,
        facets: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BuyerSearchResponse:
        """
        Search for a buyer. Please note, searches are limited to a maximum of 10,000 results.

        Parameters
        ----------
        limit : typing.Optional[int]
            A limit on the number of objects to be returned with a range between 1 and 10000. Defaults to 100.

        offset : typing.Optional[int]
            Number of results to skip before returning response. Defaults to 0.

        q : typing.Optional[str]
            Query term. The syntax for the query parameter follows elasticsearch simple query string syntax. The includes the ability to use search operators and to perform nested queries. Must be url encoded.

        filter : typing.Optional[TradeFilterList]
            Filters to be applied to search query to limit the result-set.

        facets : typing.Optional[bool]
            Whether or not to return search facets in results giving counts by field. Defaults to false.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BuyerSearchResponse

        Examples
        --------
        from sayari import Sayari
        from sayari.trade import TradeFilterList

        client = Sayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.trade.search_buyers(
            limit=1,
            filter=TradeFilterList(
                departure_country=["DEU"],
                arrival_country=["RUS"],
                hs_code=["854231"],
                arrival_date=["2024-01 TO 2024-10"],
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/trade/search/buyers",
            method="POST",
            params={
                "limit": limit,
                "offset": offset,
            },
            json={
                "q": q,
                "filter": convert_and_respect_annotation_metadata(
                    object_=filter, annotation=TradeFilterList, direction="write"
                ),
                "facets": facets,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    BuyerSearchResponse,
                    parse_obj_as(
                        type_=BuyerSearchResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequest(
                    typing.cast(
                        BadRequestResponse,
                        parse_obj_as(
                            type_=BadRequestResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise Unauthorized(
                    typing.cast(
                        UnauthorizedResponse,
                        parse_obj_as(
                            type_=UnauthorizedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowed(
                    typing.cast(
                        MethodNotAllowedResponse,
                        parse_obj_as(
                            type_=MethodNotAllowedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise RateLimitExceeded(
                    typing.cast(
                        RateLimitResponse,
                        parse_obj_as(
                            type_=RateLimitResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        InternalServerErrorResponse,
                        parse_obj_as(
                            type_=InternalServerErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncTradeClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def search_shipments(
        self,
        *,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        q: typing.Optional[str] = OMIT,
        filter: typing.Optional[TradeFilterList] = OMIT,
        facets: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ShipmentSearchResponse:
        """
        Search for a shipment. Please note, searches are limited to a maximum of 10,000 results.

        Parameters
        ----------
        limit : typing.Optional[int]
            A limit on the number of objects to be returned with a range between 1 and 10000. Defaults to 100.

        offset : typing.Optional[int]
            Number of results to skip before returning response. Defaults to 0.

        q : typing.Optional[str]
            Query term. The syntax for the query parameter follows elasticsearch simple query string syntax. The includes the ability to use search operators and to perform nested queries. Must be url encoded.

        filter : typing.Optional[TradeFilterList]
            Filters to be applied to search query to limit the result-set.

        facets : typing.Optional[bool]
            Whether or not to return search facets in results giving counts by field. Defaults to false.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ShipmentSearchResponse

        Examples
        --------
        import asyncio

        from sayari import AsyncSayari
        from sayari.trade import TradeFilterList

        client = AsyncSayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.trade.search_shipments(
                limit=1,
                filter=TradeFilterList(
                    departure_country=["DEU"],
                    arrival_country=["RUS"],
                    hs_code=["854231"],
                    arrival_date=["2024-01 TO 2024-10"],
                ),
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/trade/search/shipments",
            method="POST",
            params={
                "limit": limit,
                "offset": offset,
            },
            json={
                "q": q,
                "filter": convert_and_respect_annotation_metadata(
                    object_=filter, annotation=TradeFilterList, direction="write"
                ),
                "facets": facets,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ShipmentSearchResponse,
                    parse_obj_as(
                        type_=ShipmentSearchResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequest(
                    typing.cast(
                        BadRequestResponse,
                        parse_obj_as(
                            type_=BadRequestResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise Unauthorized(
                    typing.cast(
                        UnauthorizedResponse,
                        parse_obj_as(
                            type_=UnauthorizedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowed(
                    typing.cast(
                        MethodNotAllowedResponse,
                        parse_obj_as(
                            type_=MethodNotAllowedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise RateLimitExceeded(
                    typing.cast(
                        RateLimitResponse,
                        parse_obj_as(
                            type_=RateLimitResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        InternalServerErrorResponse,
                        parse_obj_as(
                            type_=InternalServerErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def search_suppliers(
        self,
        *,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        q: typing.Optional[str] = OMIT,
        filter: typing.Optional[TradeFilterList] = OMIT,
        facets: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SupplierSearchResponse:
        """
        Search for a supplier. Please note, searches are limited to a maximum of 10,000 results.

        Parameters
        ----------
        limit : typing.Optional[int]
            A limit on the number of objects to be returned with a range between 1 and 10000. Defaults to 100.

        offset : typing.Optional[int]
            Number of results to skip before returning response. Defaults to 0.

        q : typing.Optional[str]
            Query term. The syntax for the query parameter follows elasticsearch simple query string syntax. The includes the ability to use search operators and to perform nested queries. Must be url encoded.

        filter : typing.Optional[TradeFilterList]
            Filters to be applied to search query to limit the result-set.

        facets : typing.Optional[bool]
            Whether or not to return search facets in results giving counts by field. Defaults to false.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SupplierSearchResponse

        Examples
        --------
        import asyncio

        from sayari import AsyncSayari
        from sayari.trade import TradeFilterList

        client = AsyncSayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.trade.search_suppliers(
                limit=1,
                filter=TradeFilterList(
                    departure_country=["DEU"],
                    arrival_country=["RUS"],
                    hs_code=["854231"],
                    arrival_date=["2024-01 TO 2024-10"],
                ),
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/trade/search/suppliers",
            method="POST",
            params={
                "limit": limit,
                "offset": offset,
            },
            json={
                "q": q,
                "filter": convert_and_respect_annotation_metadata(
                    object_=filter, annotation=TradeFilterList, direction="write"
                ),
                "facets": facets,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    SupplierSearchResponse,
                    parse_obj_as(
                        type_=SupplierSearchResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequest(
                    typing.cast(
                        BadRequestResponse,
                        parse_obj_as(
                            type_=BadRequestResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise Unauthorized(
                    typing.cast(
                        UnauthorizedResponse,
                        parse_obj_as(
                            type_=UnauthorizedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowed(
                    typing.cast(
                        MethodNotAllowedResponse,
                        parse_obj_as(
                            type_=MethodNotAllowedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise RateLimitExceeded(
                    typing.cast(
                        RateLimitResponse,
                        parse_obj_as(
                            type_=RateLimitResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        InternalServerErrorResponse,
                        parse_obj_as(
                            type_=InternalServerErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def search_buyers(
        self,
        *,
        limit: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        q: typing.Optional[str] = OMIT,
        filter: typing.Optional[TradeFilterList] = OMIT,
        facets: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BuyerSearchResponse:
        """
        Search for a buyer. Please note, searches are limited to a maximum of 10,000 results.

        Parameters
        ----------
        limit : typing.Optional[int]
            A limit on the number of objects to be returned with a range between 1 and 10000. Defaults to 100.

        offset : typing.Optional[int]
            Number of results to skip before returning response. Defaults to 0.

        q : typing.Optional[str]
            Query term. The syntax for the query parameter follows elasticsearch simple query string syntax. The includes the ability to use search operators and to perform nested queries. Must be url encoded.

        filter : typing.Optional[TradeFilterList]
            Filters to be applied to search query to limit the result-set.

        facets : typing.Optional[bool]
            Whether or not to return search facets in results giving counts by field. Defaults to false.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BuyerSearchResponse

        Examples
        --------
        import asyncio

        from sayari import AsyncSayari
        from sayari.trade import TradeFilterList

        client = AsyncSayari(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.trade.search_buyers(
                limit=1,
                filter=TradeFilterList(
                    departure_country=["DEU"],
                    arrival_country=["RUS"],
                    hs_code=["854231"],
                    arrival_date=["2024-01 TO 2024-10"],
                ),
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/trade/search/buyers",
            method="POST",
            params={
                "limit": limit,
                "offset": offset,
            },
            json={
                "q": q,
                "filter": convert_and_respect_annotation_metadata(
                    object_=filter, annotation=TradeFilterList, direction="write"
                ),
                "facets": facets,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    BuyerSearchResponse,
                    parse_obj_as(
                        type_=BuyerSearchResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequest(
                    typing.cast(
                        BadRequestResponse,
                        parse_obj_as(
                            type_=BadRequestResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise Unauthorized(
                    typing.cast(
                        UnauthorizedResponse,
                        parse_obj_as(
                            type_=UnauthorizedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowed(
                    typing.cast(
                        MethodNotAllowedResponse,
                        parse_obj_as(
                            type_=MethodNotAllowedResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise RateLimitExceeded(
                    typing.cast(
                        RateLimitResponse,
                        parse_obj_as(
                            type_=RateLimitResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        InternalServerErrorResponse,
                        parse_obj_as(
                            type_=InternalServerErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)
